@@ -1,10 +1,10 @@
 ##----------------------------------------------------------------------------
 ## Module Generic - ~/lib/Module/Generic/SharedMem.pm
-## Version v0.2.4
+## Version v0.3.0
 ## Copyright(c) 2022 DEGUEST Pte. Ltd.
 ## Author: Jacques Deguest <jack@deguest.jp>
 ## Created 2021/01/18
-## Modified 2022/07/15
+## Modified 2022/07/18
 ## All rights reserved
 ## 
 ## This program is free software; you can redistribute  it  and/or  modify  it
@@ -23,7 +23,7 @@ BEGIN
     use Nice::Try;
     use Scalar::Util ();
     use JSON 4.03 qw( -convert_blessed_universally );
-    use Storable 3.25 ();
+    use Storable::Improved ();
     use constant SHM_BUFSIZ     =>  65536;
     use constant SEM_LOCKER     =>  0;
     use constant SEM_MARKER     =>  0;
@@ -109,7 +109,7 @@ EOT
             lock    => [qw( LOCK_EX LOCK_SH LOCK_NB LOCK_UN )],
             'flock' => [qw( LOCK_EX LOCK_SH LOCK_NB LOCK_UN )],
     );
-    our $VERSION = 'v0.2.4';
+    our $VERSION = 'v0.3.0';
 };
 
 # use strict;
@@ -164,6 +164,8 @@ sub attach
     return( $addr );
 }
 
+sub cbor { return( shift->_packing_method( 'cbor' ) ); }
+
 sub create { return( shift->_set_get_boolean( 'create', @_ ) ); }
 
 sub delete { return( shift->remove( @_ ) ); }
@@ -553,9 +555,19 @@ sub read
             {
                 $data = $self->_decode_json( $buffer );
             }
+            elsif( $packing eq 'cbor' )
+            {
+                $data = $self->deserialise( $buffer, serialiser => 'CBOR::XS', allow_sharing => 1 );
+            }
+            elsif( $packing eq 'sereal' )
+            {
+                $data = $self->deserialise( $buffer, serialiser => 'Sereal' );
+            }
+            # By default Storable::Improved
             else
             {
-                $data = Storable::thaw( $buffer );
+                # $data = Storable::Improved::thaw( $buffer );
+                $data = $self->deserialise( $buffer, serialiser => 'Storable::Improved' );
             }
             $self->message( 4, "Decoded data '$buffer' -> '$data': ", sub{ $self->dump( $data ) });
         }
@@ -669,6 +681,8 @@ sub reset
 
 sub semid { return( shift->_set_get_scalar( 'semid', @_ ) ); }
 
+sub sereal { return( shift->_packing_method( 'sereal' ) ); }
+
 sub serial { return( shift->_set_get_scalar( 'serial', @_ ) ); }
 
 sub shmstat
@@ -771,9 +785,23 @@ sub write
         {
             $encoded = $self->_encode_json( $data );
         }
+        elsif( $packing eq 'cbor' )
+        {
+            $encoded = $self->serialise( $data, serialiser => 'CBOR::XS', allow_sharing => 1 );
+            return( $self->pass_error ) if( !defined( $encoded ) );
+        }
+        elsif( $packing eq 'sereal' )
+        {
+            $encoded = $self->serialise( $data, serialiser => 'Sereal' );
+            return( $self->pass_error ) if( !defined( $encoded ) );
+        }
+        # Default to Storable::Improved
         else
         {
-            $encoded = Storable::freeze( $data );
+            # local $Storable::forgive_me = 1;
+            # $encoded = Storable::Improved::freeze( $data );
+            $encoded = $self->serialise( $data, serialiser => 'Storable::Improved' );
+            return( $self->pass_error ) if( !defined( $encoded ) );
         }
     }
     catch( $e )
@@ -967,6 +995,48 @@ sub DESTROY
     }
 };
 
+sub FREEZE
+{
+    my $self = CORE::shift( @_ );
+    my $serialiser = CORE::shift( @_ ) // '';
+    my $class = CORE::ref( $self );
+    my %hash  = %$self;
+    CORE::delete( @hash{ qw( addr id locked owner removed removed_semaphore semid ) } );
+    # Return an array reference rather than a list so this works with Sereal and CBOR
+    CORE::return( [$class, \%hash] ) if( $serialiser eq 'Sereal' || $serialiser eq 'CBOR' );
+    # But Storable want a list with the first element being the serialised element
+    CORE::return( $class, \%hash );
+}
+
+sub STORABLE_freeze { CORE::return( CORE::shift->FREEZE( @_ ) ); }
+
+sub STORABLE_thaw { CORE::return( CORE::shift->THAW( @_ ) ); }
+
+# NOTE: CBOR will call the THAW method with the stored classname as first argument, the constant string CBOR as second argument, and all values returned by FREEZE as remaining arguments.
+# NOTE: Storable calls it with a blessed object it created followed with $cloning and any other arguments initially provided by STORABLE_freeze
+sub THAW
+{
+    my( $self, undef, @args ) = @_;
+    my $ref = ( CORE::scalar( @args ) == 1 && CORE::ref( $args[0] ) eq 'ARRAY' ) ? CORE::shift( @args ) : \@args;
+    my $class = ( CORE::defined( $ref ) && CORE::ref( $ref ) eq 'ARRAY' && CORE::scalar( @$ref ) > 1 ) ? CORE::shift( @$ref ) : ( CORE::ref( $self ) || $self );
+    my $hash = CORE::ref( $ref ) eq 'ARRAY' ? CORE::shift( @$ref ) : {};
+    my $new;
+    # Storable pattern requires to modify the object it created rather than returning a new one
+    if( CORE::ref( $self ) )
+    {
+        foreach( CORE::keys( %$hash ) )
+        {
+            $self->{ $_ } = CORE::delete( $hash->{ $_ } );
+        }
+        $new = $self;
+    }
+    else
+    {
+        $new = CORE::bless( $hash => $class );
+    }
+    CORE::return( $new );
+}
+
 END
 {
     foreach my $id ( @$SHEM_REPO )
@@ -1046,6 +1116,45 @@ END
     sub segsz { return( shift->[SEGSZ] ); }
 
     sub uid { return( shift->[UID] ); }
+
+    sub FREEZE
+    {
+        my $self = CORE::shift( @_ );
+        my $serialiser = CORE::shift( @_ ) // '';
+        my $class = CORE::ref( $self );
+        my @array = @$self;
+        # Return an array reference rather than a list so this works with Sereal
+        CORE::return( [$class, \@array] ) if( $serialiser eq 'Sereal' );
+        # But CBOR and Storable want a list with the first element being the serialised element
+        CORE::return( $class, \@array );
+    }
+
+    sub STORABLE_freeze { CORE::return( CORE::shift->FREEZE( @_ ) ); }
+
+    sub STORABLE_thaw { CORE::return( CORE::shift->THAW( @_ ) ); }
+
+    # NOTE: CBOR will call the THAW method with the stored classname as first argument, the constant string CBOR as second argument, and all values returned by FREEZE as remaining arguments.
+    # NOTE: Storable calls it with a blessed object it created followed with $cloning and any other arguments initially provided by STORABLE_freeze
+    sub THAW
+    {
+        my( $self, undef, @args ) = @_;
+        my $ref = ( CORE::scalar( @args ) == 1 && CORE::ref( $args[0] ) eq 'ARRAY' ) ? CORE::shift( @args ) : \@args;
+        my $class = ( CORE::defined( $ref ) && CORE::ref( $ref ) eq 'ARRAY' && CORE::scalar( @$ref ) > 1 ) ? CORE::shift( @$ref ) : ( CORE::ref( $self ) || $self );
+        my $array = CORE::ref( $ref ) eq 'ARRAY' ? $ref : [];
+        # Storable pattern requires to modify the object it created rather than returning a new one
+        if( CORE::ref( $self ) )
+        {
+            @$self = @$array;
+            CORE::return( $self );
+        }
+        else
+        {
+            my $new = bless( $array => $class );
+            CORE::return( $new );
+        }
+    }
+
+    sub TO_JSON { CORE::return( [ @{$_[0]} ] ); }
 }
 
 # NOTE: Module::Generic::SemStat class
@@ -1099,6 +1208,45 @@ END
     sub otime { return( shift->[OTIME] ); }
 
     sub uid { return( shift->[UID] ); }
+
+    sub FREEZE
+    {
+        my $self = CORE::shift( @_ );
+        my $serialiser = CORE::shift( @_ ) // '';
+        my $class = CORE::ref( $self );
+        my @array = @$self;
+        # Return an array reference rather than a list so this works with Sereal
+        CORE::return( [$class, \@array] ) if( $serialiser eq 'Sereal' );
+        # But CBOR and Storable want a list with the first element being the serialised element
+        CORE::return( $class, \@array );
+    }
+
+    sub STORABLE_freeze { CORE::return( CORE::shift->FREEZE( @_ ) ); }
+
+    sub STORABLE_thaw { CORE::return( CORE::shift->THAW( @_ ) ); }
+
+    # NOTE: CBOR will call the THAW method with the stored classname as first argument, the constant string CBOR as second argument, and all values returned by FREEZE as remaining arguments.
+    # NOTE: Storable calls it with a blessed object it created followed with $cloning and any other arguments initially provided by STORABLE_freeze
+    sub THAW
+    {
+        my( $self, undef, @args ) = @_;
+        my $ref = ( CORE::scalar( @args ) == 1 && CORE::ref( $args[0] ) eq 'ARRAY' ) ? CORE::shift( @args ) : \@args;
+        my $class = ( CORE::defined( $ref ) && CORE::ref( $ref ) eq 'ARRAY' && CORE::scalar( @$ref ) > 1 ) ? CORE::shift( @$ref ) : ( CORE::ref( $self ) || $self );
+        my $array = CORE::ref( $ref ) eq 'ARRAY' ? $ref : [];
+        # Storable pattern requires to modify the object it created rather than returning a new one
+        if( CORE::ref( $self ) )
+        {
+            @$self = @$array;
+            CORE::return( $self );
+        }
+        else
+        {
+            my $new = bless( $array => $class );
+            CORE::return( $new );
+        }
+    }
+
+    sub TO_JSON { CORE::return( [ @{$_[0]} ] ); }
 }
 
 1;
